#!/usr/local/bin/perl

#
#	This programm starts an etherHistory Collector and displays
#	the data this collector collects.
#	It should be called as:
#
#	etherstats.pl <iface> <buckets> <interval>[<host> <community> <medium>]
#
#	The algoritme that should be used would be:
#
#		wait for a reasonable period of time (related 
#		to <buckets>*<interval>)
#
#		retreive all the buckets that have been filled
#		until then
#
#		loop
#
require "spook.pl";	   
require "collector.pl";
#
# handle the arguments
#
if( @ARGV<3 || @ARGV>6 )
{
	die 	"call as:  perl etherhist.pl <iface> <buckets> <interval> [<host> <community> <medium>]\n" ,
		" where:\n" ,
		"	<iface>		iface-number of ethernet interface in RMON monitor\n",
		"	<buckets>	number of samples after which monitor will wrap\n" ,
		" 	<interval>	interval (secs) between two samples/buckets\n" ,
		"	<host>		name of RMON monitor\n" ,
		"	<community>	community name for creating collectors\n",
		"	<medium>	UDP or PIPE\n";
}
($iniIface, $iniBuck, $iniInt ,$iniHost , $iniComm , $iniMed,@rest) = @ARGV;

# create the SPOOK connection
&snmp_open(1) || die "cant open a Spook connection\n";
if($iniHost ne "") { &snmp_host(1,$iniHost);}
if($iniMed ne "")  {&snmp_medium(1,$iniMed);}
&snmp_connect(1) || die "could't connect to ($iniHost,$iniMed)";
if($iniComm ne "") {&snmp_community(1,$iniComm);}

#
# function iniFun will be called during initialisation
# of the collector. This is the place to set all kind
# of collector specific values. In the case of etherStats, 
# this is only the ethernet-interface number etherStatsDataSource
#
sub iniFun
{
	local($connNr , $collName , $collNr) = @_;
	local($varName , $res , $iface , $int);
	
	$int = $iniInt ;
	
	# the historyControl collector requires that 	
	# dataSource should be set to the active device number,
	# Interval to time between samples (1/100 secs)
	# Buckets to number of buckets to be used
	$varName = $collName . "DataSource\[$collNr\]";
	#
	# yes we know, very ugly
	$iface = "1.3.6.1.2.1.2.2.1.1." . $iniIface;
	$res = &snmp_set($connNr,"$varName=$iface");
	if($res != $iface)
	{
		return 0;
	}
	$res = &snmp_set($connNr,$collName . "Interval\[$collNr\]=$int");
	if($res != $int)
	{
		return 0;
	}
 	$res = &snmp_set($connNr,$collName . "BucketsRequested\[$collNr\]=$iniBuck");
	if($res != $iniBuck)
	{
		return 0;
	}
	return 1;
}	   


# create the etherStats collector with the correct inFun
$connNr = &create_new_collector(1,"historyControl","eikel",main'iniFun) || die "failed creating\n";
print STDERR "created etherStats collector $connNr\n";

#prepare the signal handler
sub sigHandler
{
	local($sig) = @_;
	print("\nTerminating etherstats collector and exiting\n");
	# remove the etherStats collector
	&remove_collector(1,"historyControl",$connNr,"eikel") || die "failed removing\n";
	
	# close the Spook connection
	&snmp_disconnect(1);
	&snmp_close(1);
	exit;
}
$SIG{'INT'}  = 'sigHandler';
$SIG{'QUIT'} = 'sigHandler';

       

$bucket=0;
$period = $iniInt * ($iniBuck -1);
$pint = 24;
while(1)
{	
	#wait for a given period of time
	sleep($period);		# sleep for the period w
	
	# collect and print all filled buckets since the last try
	# starting from the last bucket collected
	@resp = &snmp_getnext(1,"etherHistorySampleIndex[$connNr.$bucket]");
	while($resp[0] eq "etherHistorySampleIndex")
	{
		$bucket = $resp[2];
		@req = (
			"etherHistorySampleIndex[$connNr.$bucket]",
			"etherHistoryIntervalStart[$connNr.$bucket]",
			"etherHistoryDropEvents[$connNr.$bucket]",
			"etherHistoryOctets[$connNr.$bucket]",
			"etherHistoryUtilization[$connNr.$bucket]",
			"etherHistoryPkts[$connNr.$bucket]",
			"etherHistoryBroadcastPkts[$connNr.$bucket]",
			"etherHistoryMulticastPkts[$connNr.$bucket]",
		);
		@res = &snmp_get(1,@req);
		# check if banner should be printed  
		if($pint >= 23)
		{
			printf(STDERR "\n%08s %08s %08s %08s %08s %08s %08s %08s %08s\n",
				"Bucket",
				"Start",
				"Drops",
				"Octets",
				"Util",
				"Pkts",
				"Brdcst",
				"Mltcst"
				);
			$pint = 0;
		}
		$pint++;
		for($j=0;$j<@res;$j++)
		{
			printf("%8d ",$res[$j]);
		}
		print "\n";
		@resp = &snmp_getnext(1,"etherHistorySampleIndex[$connNr.$bucket]");
	}
}

