#
# This Perl library contains utility functions to manipulate RMON output.
# Most routines are conversionlike to make output more human-readable.
#

# convert a hex representation of an IP address to a decimal form
sub iph2ipa
{
    local($iph) = @_;
    local($d1,$d2,$d3,$d4);

    ($d1,$d2,$d3,$d4) = split(/:/,$iph);
    return( join('.',hex($d1),hex($d2),hex($d3),hex($d4)));
}

#convert a decimal representation of an IP address to a hex form
sub ipa2iph
{
    local($ipa) = @_;
    local($d1,$d2,$d3,$d4);
    
    ($d1,$d2,$d3,$d4) = split(/\./,$ipa);
    return( sprintf("%02x:%02x:%02x:%02x",$d1,$d2,$d3,$d4));
}			

# get the real name of an ip address
sub ip2name
{
    local($addr) = @_;
    local($n,$name,$aliases,$addrtype,$length,@addrs);

    if( $addr =~ /:/)
    {
	$addr = &iph2ipa($addr);
    }
    $n = pack("C4",split(/\./,$addr));
    ($name,$aliases,$addrtype,$length,@addrs) =gethostbyaddr($n,2);
    if( $name eq "")
    {
	return($addr);
    }
    return($name);
}

# get the service name
sub serv2name
{
    local($port,$proto) = @_;
    local($name,$aliases,$p,$pr);
    ($name,$aliases,$p,$pr) = getservbyport($port,$proto);
    if($name eq "")
    {
	return($port);
    }
    return($name);
}

1;
#$test1 = "c0:57:64:0d";
#$test2 = "192.87.100.13";
#print $test1 . " \n";
#print &iph2ipa($test1) ." \n";
#print &ipa2iph($test2) ." \n";
#print &ipa2iph(&iph2ipa($test1)) ." \n";
#print &ip2name($test1) . " \n";
#print &ip2name($test2) . " \n";

# print &serv2name(23,'tcp');
